package support.redhat.entity.association;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import support.redhat.util.AutoCloser;

import junit.framework.TestCase;

import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import support.redhat.entity.association.ParentEntity;
import support.redhat.entity.association.EmbeddedEntity_;
import support.redhat.entity.association.ParentEntity_;

public class TestHibernate extends TestCase {
	private EntityManager em;

	private final AutoCloser m_closer = new AutoCloser();

	@Override
	protected void setUp() throws Exception {
		try {
			// Configures settings from annotations + persistence.xml
			EntityManagerFactory emf = Persistence.createEntityManagerFactory("TEST");
			m_closer.push(emf);
			em = emf.createEntityManager();
			m_closer.push(em);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	@Override
	protected void tearDown() throws Exception {
		if (em != null) {
			m_closer.close();
			em = null;
		}
	}

	public void testCriteria() {
		CriteriaBuilder cb = em.getCriteriaBuilder();

		CriteriaQuery<ParentEntity> cq = cb.createQuery(ParentEntity.class);
		Root<ParentEntity> root = cq.from(ParentEntity.class);

		root.fetch(ParentEntity_.embedded).fetch(EmbeddedEntity_.children);
		cq.where(cb.equal(root.get(ParentEntity_.guid), "P1"));

		TypedQuery<ParentEntity> typeQuery = em.createQuery(cq);

		System.out.println("############## JPQL ################################################################");
		System.out.println(typeQuery.unwrap(org.hibernate.Query.class).getQueryString());
		System.out.println("##############@@@@@@################################################################");

		RootLoggerFilteredInterceptor interceptor = new RootLoggerFilteredInterceptor("org.hibernate.engine.jdbc.spi.SqlStatementLogger");
		System.out.println("############## SQL #################################################################");
		interceptor.attach();
		ParentEntity result = typeQuery.getSingleResult();
		interceptor.detach();
		System.out.println("##############@@@@@@################################################################");
		if (!interceptor.capturedContainsString("TEST_CHILD", true)) {
			fail("Should have executed JOIN on TEST_CHILD\nActual Query: ["
				+ interceptor.getCapturedEvents().get(0).getMessage() + "]");
		}

		// The below would result in an extra implicit query to get the children that should have been join fetched above
		assertNotNull(result.getEmbedded().getChildren());
		assertEquals(6, result.getEmbedded().getChildren().size());
	}

	public void XtestSql() {
		TypedQuery<ParentEntity> typeQuery = em.createQuery("SELECT parent FROM ParentEntity as parent "
			+ "LEFT JOIN FETCH parent.embedded.children "
			+ "WHERE parent.guid= 'P1'", ParentEntity.class);

		System.out.println("############## JPQL ################################################################");
		System.out.println(typeQuery.unwrap(org.hibernate.Query.class).getQueryString());
		System.out.println("##############@@@@@@################################################################");

		System.out.println("############## SQL #################################################################");
		ParentEntity result = typeQuery.getSingleResult();
		System.out.println("##############@@@@@@################################################################");

		assertNotNull(result.getEmbedded().getChildren());
		assertEquals(6, result.getEmbedded().getChildren().size());
	}

	// Not thread safe - dangerous if using async logging and the "user interface" methods (attach, detach, etc.) aren't thread safe either
	public class RootLoggerFilteredInterceptor extends org.apache.log4j.AppenderSkeleton {
		private final String m_sourceClass;
		private final java.util.ArrayList<org.apache.log4j.spi.LoggingEvent> m_captured = new java.util.ArrayList<org.apache.log4j.spi.LoggingEvent>();

		public RootLoggerFilteredInterceptor(String sourceClass) {
			super();
			m_sourceClass = sourceClass;
		}

		// Add an appender to the root logger to capture logging for the sourceClass
		public void attach() {
			m_captured.clear(); // starting up (again)
			org.apache.log4j.Logger.getRootLogger().addAppender(this);
		}

		// Remove the appender from the root logger to avoid further capture
		public void detach() {
			org.apache.log4j.Logger.getRootLogger().removeAppender(this);
		}

		// Get all events captured since the last attach
		public java.util.ArrayList<org.apache.log4j.spi.LoggingEvent> getCapturedEvents() {
			return m_captured;
		}

		// Check for the presence of some string in the message text captured since the last attach
		public boolean capturedContainsString(String string) {
			return capturedContainsString(string, false);
		}

		// Check for the presence of some string in the message text captured since the last attach
		public boolean capturedContainsString(String string, boolean ignoreCase) {
			for (org.apache.log4j.spi.LoggingEvent event : getCapturedEvents()) {
				if (ignoreCase && event.getMessage().toString().toLowerCase().contains(string.toLowerCase())) {
					return true;
				} else if (event.getMessage().toString().contains(string)) {
					return true;
				}
			}
			return false;
		}

		// Does the actual intercept, implementing the abstract method from org.apache.log4j.AppenderSkeleton
		@Override
		protected void append(org.apache.log4j.spi.LoggingEvent event) {
			if (m_sourceClass.equals(event.getLocationInformation().getClassName())) {
				m_captured.add(event);
			}
		}

		// Required appender interface method
		public boolean requiresLayout() {
			// Not sure what this really means
			return false;
		}

		// Required appender interface method
		public void close() {
			// not opening any resources, so nothing to do
		}
	}
}
